#ifndef GRAPHICS_H
#define GRAPHICS_H


#include <stdio.h>
using namespace std;


//Screen dimension constants
const int SCREEN_WIDTH = 575;
const int SCREEN_HEIGHT = 469;

//Starts up SDL and creates window
bool init();

//Loads media
bool loadMedia();

//Frees media and shuts down SDL
void close();

//The window we'll be rendering to
SDL_Window* gWindow = NULL;



//The surface contained by the window
SDL_Surface* gScreenSurface = NULL;

//The image we will load and show on the screen
SDL_Surface* gchess = NULL;
SDL_Surface* gp=NULL;
SDL_Surface* gb=NULL;
SDL_Surface* lm=NULL;
//Console print
void printBoard(piece* sq[8][8])
{
    gprint(sq);

}
bool init()
{
	//Initialization flag
	bool success = true;

	//Initialize SDL
	if( SDL_Init( SDL_INIT_VIDEO ) < 0 )
	{
		printf( "SDL could not initialize! SDL_Error: %s\n", SDL_GetError() );
		success = false;
	}
	else
	{
		//Create window
		gWindow = SDL_CreateWindow( "CHESS", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, SCREEN_WIDTH, SCREEN_HEIGHT, SDL_WINDOW_SHOWN );
		if( gWindow == NULL )
		{
			printf( "Window could not be created! SDL_Error: %s\n", SDL_GetError() );
			success = false;
		}
		else
		{
			//Get window surface
			gScreenSurface = SDL_GetWindowSurface( gWindow );
		}
	}

	return success;
}


bool loadMedia()
{
	//Loading success flag
	bool success = true;

	//Load splash image
	gp=SDL_LoadBMP("img/pieces.bmp");
	gchess = SDL_LoadBMP( "img/chess.bmp" );
	gb = SDL_LoadBMP( "img/box.bmp" );
	lm = SDL_LoadBMP("img/lastmove.bmp");
	if( gchess == NULL )
	{
		printf( "Unable to load image %s! SDL Error: %s\n", "img/chess.bmp", SDL_GetError() );
		success = false;
	}
	if( gp == NULL )
	{
		printf( "Unable to load image %s! SDL Error: %s\n", "img/pieces.bmp", SDL_GetError() );
		success = false;
	}
	else{
        SDL_SetColorKey( gp, SDL_TRUE, SDL_MapRGB( gp->format, 0, 0xFF, 0xFF ) );
	}
	if( gb == NULL )
	{
		printf( "Unable to load image %s! SDL Error: %s\n", "img/box.bmp", SDL_GetError() );
		success = false;
	}
	else{
        SDL_SetColorKey( gb, SDL_TRUE, SDL_MapRGB( gb->format, 0, 0xFF, 0xFF ) );
	}
    if( lm == NULL )
	{
		printf( "Unable to load image %s! SDL Error: %s\n", "img/lastmove.bmp", SDL_GetError() );
		success = false;
	}
	else{
        SDL_SetColorKey( lm, SDL_TRUE, SDL_MapRGB( lm->format, 0, 0xFF, 0xFF ) );
	}

	return success;
}

void close()
{
	//Deallocate surface
	SDL_FreeSurface( gchess );
	gchess = NULL;
    SDL_FreeSurface(gp);
    SDL_FreeSurface(gb);
    SDL_FreeSurface(lm);
    lm = NULL;
    gp=NULL;
    gb=NULL;
	//Destroy window
	SDL_DestroyWindow( gWindow );
	gWindow = NULL;

	//Quit SDL subsystems
	SDL_Quit();
}
void initialize(){
    if( !init() )
    {
		printf( "Failed to initialize!\n" );
    }
	else
    {
		//Load media
		if( !loadMedia() )
		{
			printf( "Failed to load media!\n" );
		}
	}
}
void gprint(piece *sq[8][8])
{
    SDL_BlitSurface( gchess, NULL, gScreenSurface, NULL );
    SDL_Rect src;
    SDL_Rect dest;
    dest.w=50;
    dest.h=50;
    src.h=50;
    src.w=50;
	for(int i=0;i<8;i++){
        for(int j=0; j<8;j++){
            if(sq[i][j]!=NULL){
                dest.y=37+50*i;
                dest.x=13+50*j;
                switch(sq[i][j]->getName()){
                case 'P':
                    src.x=0;
                    break;
                case 'B':
                    src.x=50;
                    break;
                case 'N':
                    src.x=100;
                    break;
                case 'R':
                    src.x=150;
                    break;
                case 'Q':
                    src.x=200;
                    break;
                case 'K':
                    src.x=250;
                    break;
                }
                if(sq[i][j]->getColor()=='W'){
                    src.y=0;
                }
                else{
                    src.y=50;
                }
                SDL_BlitSurface( gp, &src, gScreenSurface, &dest );
            }
        }
	}
	SDL_UpdateWindowSurface(gWindow);

}

void getMove(int move[], int playerTurn, piece* sq[8][8], int moveHistory[5][8])
{
    if(playerTurn == human)
    {
        //--------Graphical input----------//

        //Main loop flag
        //bool quit = false;
        //Event handler
        SDL_Event e;
        int moveTwo=0;
        int mx;
        int my;
        //While application is running
        while( !quit ) {
            while( SDL_PollEvent( &e ) != 0 )
                {
                    //User requests quit
                    switch( e.type){
                        case SDL_QUIT: quit = true;
                            break;
                        case SDL_MOUSEBUTTONDOWN:
                                switch(e.button.button){
                                    case SDL_BUTTON_LEFT:
                                        if(mx>=450&&mx<=550){
                                            if(my>=210&&my<=260){
                                                quit=true;
                                            }
                                            if(my>=155&&my<=205)
                                                {
                                                reset=true;
                                                move[0]=-1;
                                                return;
                                                }
                                        }
                                        if((mx-13)/50>=0 && (my-37)/50>=0 && (mx-13)/50<8 && (my-37)/50<8 &&
                                           (sq[(my-37)/50][(mx-13)/50]!=NULL || moveTwo==1)){
                                                if(moveTwo==0){
                                                    if(sq[(my-37)/50][(mx-13)/50]->getPlayer() != playerTurn){
                                                        continue;
                                                    }
                                                    SDL_Rect bxs;
                                                    SDL_Rect bxd;
                                                    bxs.h=50;
                                                    bxd.h=50;
                                                    bxs.w=50;
                                                    bxd.w=50;
                                                    bxs.x=0;
                                                    bxs.y=0;
                                                    bxd.x=13+((mx-13)/50)*50;
                                                    bxd.y=37+((my-37)/50)*50;
                                                    SDL_BlitSurface(gb,&bxs,gScreenSurface,&bxd);
                                                    SDL_UpdateWindowSurface(gWindow);
                                                }
                                                    move[moveTwo*2]=(mx-13)/50;
                                                    move[moveTwo*2 + 1]=(my-37)/50;
                                                    moveTwo++;
                                        }
                                        if(moveTwo==2){
                                            if(move[0]==4&&move[1]==7&&
                                               move[2]==6&&move[3]==7&&sq[7][4]->getName()=='K'){
                                                   move[2]=0;
                                                   move[3]=9;
                                            }
                                            if(move[0]==4&&move[1]==7&&
                                               move[2]==2&&move[3]==7&&sq[7][4]->getName()=='K'){
                                                   move[2]=1;
                                                   move[3]=9;
                                            }
                                            if(move[0]==4&&move[1]==0&&
                                               move[2]==6&&move[3]==0&&sq[0][4]->getName()=='K'){
                                                   move[2]=0;
                                                   move[3]=9;
                                            }
                                            if(move[0]==4&&move[1]==0&&
                                               move[2]==2&&move[3]==0&&sq[0][4]->getName()=='K'){
                                                   cout<<"in";
                                                   move[2]=1;
                                                   move[3]=9;
                                            }
                                            return;
                                        }
                            break;
                                }
                        case SDL_MOUSEMOTION:
                            mx=e.motion.x;
                            my=e.motion.y;
                            break;
                    }
                }
        }

        move[0]=-1;
        return;
    }
    else
    {
        getCompMove(move, sq, moveHistory);
    }

    }
void gPromote(int r,int c, piece* sq[8][8],int playerTurn){
    SDL_Surface* promotion;
    promotion = SDL_LoadBMP( "img/promotion.bmp" );
    if( promotion == NULL )
	{
		printf( "Unable to load image %s! SDL Error: %s\n", "img/promotion.bmp", SDL_GetError() );
	}
	else{
        SDL_SetColorKey( promotion, SDL_TRUE, SDL_MapRGB( promotion->format, 0, 0xFF, 0xFF ) );
	}
	SDL_Rect dest;
	dest.x=425;
	dest.y=0;
	SDL_BlitSurface(promotion,NULL,gScreenSurface,&dest);
	SDL_UpdateWindowSurface(gWindow);
	SDL_Event ev;
	while( true ) {
            while( SDL_PollEvent( &ev ) != 0 )
                {
                    //User requests quit
                    int mx;
                    int my;
                    switch( ev.type){
                        case SDL_MOUSEBUTTONDOWN:
                                switch(ev.button.button){
                                    case SDL_BUTTON_LEFT:
                                        if(mx>=450&&mx<=500&&my>=177&&my<=227){
                                            sq[r][c]=new bishop((playerTurn==0)?('W'):('B'),playerTurn);

                                        }
                                        else if(mx>=500&&mx<=550&&my>=177&&my<=227){
                                            sq[r][c]=new knight((playerTurn==0)?('W'):('B'),playerTurn);

                                        }
                                        else if(mx>=450&&mx<=500&&my>=227&&my<=277){
                                            sq[r][c]=new rook((playerTurn==0)?('W'):('B'),playerTurn);

                                        }
                                        else if(mx>=500&&mx<=550&&my>=227&&my<=277){
                                            sq[r][c]=new queen((playerTurn==0)?('W'):('B'),playerTurn);

                                        }
                                        else{
                                            continue;
                                        }
                                        SDL_UpdateWindowSurface(gWindow);
                                        return;

                            break;
                                }
                        case SDL_MOUSEMOTION:
                            mx=ev.motion.x;
                            my=ev.motion.y;
                            break;
                    }
                }
        }
    //SDL_Delay(3000);
    SDL_FreeSurface(promotion);
    promotion=NULL;

}
int ShowMessage(char title[], char messages[], char b1[], char b2[]){

    const SDL_MessageBoxButtonData buttons[] = {
        { /* .flags, .buttonid, .text */        0, 0, b2 },
        { SDL_MESSAGEBOX_BUTTON_RETURNKEY_DEFAULT, 1, b1 },
    };
    const SDL_MessageBoxColorScheme colorScheme = {
        { /* .colors (.r, .g, .b) */
            /* [SDL_MESSAGEBOX_COLOR_BACKGROUND] */
            { 255,   0,   0 },
            /* [SDL_MESSAGEBOX_COLOR_TEXT] */
            {   0, 255,   0 },
            /* [SDL_MESSAGEBOX_COLOR_BUTTON_BORDER] */
            { 255, 255,   0 },
            /* [SDL_MESSAGEBOX_COLOR_BUTTON_BACKGROUND] */
            {   0,   0, 255 },
            /* [SDL_MESSAGEBOX_COLOR_BUTTON_SELECTED] */
            { 255,   0, 255 }
        }
    };
    const SDL_MessageBoxData messageboxdata = {
        SDL_MESSAGEBOX_INFORMATION, /* .flags */
        NULL, /* .window */
        title, /* .title */
        messages, /* .message */
        SDL_arraysize(buttons), /* .numbuttons */
        buttons, /* .buttons */
        &colorScheme /* .colorScheme */
    };
    int buttonid;
    if (SDL_ShowMessageBox(&messageboxdata, &buttonid) < 0) {
        SDL_Log("error displaying message box");
        return -1;
    }
    if (buttonid == -1) {
        SDL_Log("no selection");
    }
    else if (buttonid==0){
        return 0;
    }
    else if (buttonid==1){
        return 1;
    }
    return 1;
}
#endif
